import { defHttp, otherHttp } from '/@/utils/http/axios'
import { LoginParams, GetUserInfoModel } from './model/userModel'

import { ErrorMessageMode } from '/#/axios'

enum Api {
    Login = 'mgr/api/porxy/login',
    GetUserInfo = 'mgr/api/porxy/users',
    AddUser = 'mgr/api/porxy/createUser',
    DeleteUser = 'mgr/api/porxy/delUser',
    SetUserPower = 'mgr/api/porxy/setAuth',
    ModifyUserPwd = 'mgr/api/porxy/mdfPwd',
}
interface UserInfoParams {
    user: string,
    token: string,
}

interface AddUserParams {
    user: string,
    pwd: string,
}

interface DeleteUserParams {
    user: string,
}

export interface SetUserPowerParams {
    user: string,
    auth: UserPowerItem
}

interface UserPowerItem {
    user_mgr?: number,
    auth_mgr?: number,
    db_mgr?: number
}

interface ModifyUserPwdParams{
    user: string,
    pwd: string,
}

/**
 * @description: 登录管理端
 */
export function loginApi(params: LoginParams, mode: ErrorMessageMode = 'modal') {
    return otherHttp.get(
        {
            url: Api.Login,
            params,
        },
        {
            errorMessageMode: mode,
        },
    )
}

/**
 * @description: 获取用户信息
 */
export function getUserInfo(params: UserInfoParams) {
    if (params) {
        return otherHttp.get({ url: Api.GetUserInfo, headers: { 'Authorization': 'Basic ' + btoa(params.user + ':' + params.token) } }, { errorMessageMode: 'none' })
    } else {
        return otherHttp.get({ url: Api.GetUserInfo }, { errorMessageMode: 'none' })
    }
}

/**
 * @description: 增加用户
 */
export function addUser(params: AddUserParams, headers: UserInfoParams) {
    return otherHttp.get({ url: Api.AddUser, params, headers: { 'Authorization': 'Basic ' + btoa(headers.user + ':' + headers.token) } }, { errorMessageMode: 'none' })
}

/**
 * @description: 删除用户
 */
export function deleteUser(params: DeleteUserParams, headers: UserInfoParams) {
    return otherHttp.get({ url: Api.DeleteUser, params, headers: { 'Authorization': 'Basic ' + btoa(headers.user + ':' + headers.token) } }, { errorMessageMode: 'none' })
}

/**
 * @description: 设置权限
 */
export function setUserPower(params: SetUserPowerParams, headers: UserInfoParams) {
    return otherHttp.post({ url: Api.SetUserPower, params, headers: { 'Authorization': 'Basic ' + btoa(headers.user + ':' + headers.token) } }, { errorMessageMode: 'none' })
}

/**
 * @description: 修改密码
 */
export function modifyUserPwd(params: ModifyUserPwdParams, headers: UserInfoParams) {
    return otherHttp.get({ url: Api.ModifyUserPwd, params, headers: { 'Authorization': 'Basic ' + btoa(headers.user + ':' + headers.token) } }, { errorMessageMode: 'none' })
}