import { DbItem, DataR, Select, Data } from "../structs/rpc/db_mgr.struct";
import { TabDecode, TabEncode } from "../net/serialization/tab_meta";
import { BonBuffer } from "../net/serialization/bon";
import { EnumType, Type } from "../net/serialization";
import { AUTH, AuthDb, AuthDbKey, FM, UserDb } from "../structs/db/user_db.struct";
import { hex_md5 } from "./md5";
import { clientRpc } from "../net/index";
import { checkNum } from "./tools";

const request = async (topic: string, data?: any) => {
    let r = await clientRpc(topic, data);
    
    if (r instanceof Error) {
        throw r
    }
    return r;
}

// 写数据
export const dbWrite = async (ware: string, tab: string, key: any, value: any) => {
    let meta = await tabInfo(ware, tab);
    let write = new DbItem();
    write.tab = tab;
    write.ware = ware;

    // 处理key
    let kBon = new BonBuffer();
    new TabEncode(meta.k, key).bonEncode(kBon);
    write.key = kBon.getBuffer();
    // 处理value
    if (!(value === null || value === undefined)) {
        let vBon = new BonBuffer();
        new TabEncode(meta.v, value).bonEncode(vBon);
        write.value = vBon.getBuffer();
    }

    return await request('management_page/server/rpc/db_mgr.write', write);
}

// 读数据
export const dbRead = async (ware: string, tab: string, key: any) => {
    //key做简单处理 默认为简单主键字符串或者数字
    let meta = await tabInfo(ware, tab);
    if (checkNum(Type[meta.k.type])) key = +key;
    let item = new DbItem();
    item.tab = tab;
    item.ware = ware;
    // 处理key
    let kBon = new BonBuffer();
    new TabEncode(meta.k, key).bonEncode(kBon);
    item.key = kBon.getBuffer();
    let r = <DataR>await request('management_page/server/rpc/db_mgr.read', item);
    item.key = new TabDecode(meta.k).bonDecode(new BonBuffer(r.datas[0].key));
    item.value = new TabDecode(meta.v).bonDecode(new BonBuffer(r.datas[0].value));

    return item;
}

export const allTabs = (): Promise<any> => {
    let logfile = request('management_page/server/rpc/db_mgr.getAllTables', "logfile");
    let memory = request('management_page/server/rpc/db_mgr.getAllTables', "memory");
    return Promise.all([logfile, memory]).then(([l, m]) => {
        l.forEach(item => item.ware = 'logfile');
        m.forEach(item => item.ware = 'memory');
        return l.concat(m);
    });
}
// 获取所有表
export const tabs = async () => {

}

// 获取表大小
export const tabSize = async (ware: string, tab: string) => {
    let item = new DbItem();
    item.tab = tab;
    item.ware = ware;
    return await request('management_page/server/rpc/db_mgr.getTableSize', item);
}

// 获取表信息
export const tabInfo = async (ware: string, tab: string): Promise<{ k: EnumType, v: EnumType }> => {
    let info = new DbItem();
    info.ware = ware;
    info.tab = tab;
    let r = await request('management_page/server/rpc/db_mgr.getTableInfo', info);
    if (typeof r === "string") {
        return JSON.parse(r);
    } else {
        console.log('get tabinfo error:', r);
    }

};

// 登录
export const loginFunc = async (user: string, pwd: string) => {
    let userDB = new UserDb();
    userDB.user = user;
    userDB.pwd = hex_md5(pwd);
    return request('management_page/server/rpc/user.login', userDB);
}

// 创建用户
export const createUserFunc = async (user: string, pwd: string) => {
    let userDB = new UserDb();
    userDB.user = user;
    userDB.pwd = hex_md5(pwd);
    return request('management_page/server/rpc/user.createUser', userDB);
}

// 删除用户
export const delUserFunc = async (user: string) => {
    return request('management_page/server/rpc/user.delUser', user);
}

// 修改密码
export const mdfPwdFunc = async (user: string, pwd: string) => {
    let userDB = new UserDb();
    userDB.user = user;
    userDB.pwd = hex_md5(pwd);
    return request('management_page/server/rpc/user.mdfPwd', userDB);
}

// 获取所有用户
export const usersFunc = async () => {
    return request('management_page/server/rpc/user.users');
}

// 设置权限
export const setAuthFunc = async (user: string, fm: FM, auth: AUTH) => {
    let authDB = new AuthDb();
    let key = new AuthDbKey();
    key.id = user;
    key.module = fm;
    authDB.key = key;
    authDB.auth = auth;

    return request('management_page/server/rpc/user.setAuth', authDB);
}

// 筛选表信息
export const selectTable = async (ware: string, tab: string, pageSize: number, key: any, descending: boolean = false) => {
    let meta = await tabInfo(ware, tab);
    let select = new Select();
    let item = new DbItem();
    console.log(item);

    item.ware = ware;
    item.tab = tab;

    if (!(key === null || key === undefined)) {
        let kBon = new BonBuffer();
        new TabEncode(meta.k, key).bonEncode(kBon);
        item.key = kBon.getBuffer();
    }
    select.item = item;
    select.page_count = pageSize;
    select.descending = descending;
    let r: DataR = await request('management_page/server/rpc/db_mgr.select', select);
    let datas = [];
    r.datas.forEach((data: Data) => {
        // let key = new TabDecode(meta.k).bonDecode(new BonBuffer(data.key));
        // let value = new TabDecode(meta.v).bonDecode(new BonBuffer(data.value));
        let key, value;
        try {
            key = new TabDecode(meta.k).bonDecode(new BonBuffer(data.key));
            value = new TabDecode(meta.v).bonDecode(new BonBuffer(data.value));
        } catch (error) {
            key = new BonBuffer(data.key).read();
            value = new TabDecode(meta.v).bonDecode(new BonBuffer(data.value));
        }
        datas.push([key, value]);
    })
    return datas;
}

// 添加一条信息
export const addData = async (ware: string = "logfile", tab: string = 'mqttrpc_demo/server/test_db.TestDB7') => {
    let map = new Map();
    map.set("1", "2");
    let data = {
        key: Date.now(),
        _bool: true,
        _u8: 1,
        _u16: 1,
        _u32: 1,
        _usize: 1,
        _i8: 1,
        _i16: 1,
        _i32: 1,
        _isize: 1,
        _f32: 0.1,
        _f64: 0.2,
        _str: "xxxx",
        _arr: ['1', '2', '3', '1', '2', '3', '1', '2', '3', '1', '2', '3', '1', '2', '3'],
        _map: map
    };
    await dbWrite(ware, tab, data.key, data);
    return await dbRead(ware, tab, data.key);
}

// 添加一条信息
export const addData1 = async (ware: string = "logfile", tab: string = 'mqttrpc_demo/server/test_db.TestDB7', data: any) => {
    const key = data[Object.keys(data)[0]];
    await dbWrite(ware, tab, key, data);
    return await dbRead(ware, tab, key);
}
