

import { Entrance } from "pi_ucore/event/entrance.struct"
import { Env } from "pi_ucore/db/env";
import { Log } from "pi_pt/util/log";
import { Tr } from "pi_ucore/db/mgr";
import { getLatestWorkerVersion } from "./gray_lib";


declare var env: Env;

// 事件类型定义
export interface EventData {
    name: string;
    plugin_name: string;
    auth: string;
    version: string;
    note: Map<string, string>;
}

let eventMap = new Map<string, EventData[]>();

//获取所有插件名
export const pluginsList = async (gray?: string): Promise<string[]> => {
    gray = gray || getLatestWorkerVersion() + "";
    const events = await getData(gray);
    let plugins = new Set<string>();
    for (let elem of events) {
        plugins.add(elem.plugin_name);
    }
    return [...plugins];
}

// 获取指定插件
export const getPlugin = async (name: string, gray?: string): Promise<EventData | null> => {
    gray = gray || getLatestWorkerVersion() + "";
    let events = await getData(gray);
    for (let elem of events) {
        if (elem.name === name) {
            return elem;
        }
    }
    return null;
}

const getData = async (gray: string): Promise<EventData[]> => {
    // 获取eventMap中是否存在gray，不存在则初始化
    if (!eventMap.has(gray)) {
        let events = [];
        await env.dbMgr.read(async (tr: Tr) => {
            let it = tr.kvList("memory", getCfgName(Entrance._$info.name, gray), null, false, "");
            for await (let elem of it) {
                let entrance = elem[1] as Entrance;
                const pluginName = entrance.note.get("management_plugin_name");
                if (pluginName) {
                    let eventData: EventData = {
                        name: entrance.note.get("event"),
                        plugin_name: pluginName,
                        auth: entrance.note.get("auth"),
                        version: gray,
                        note: entrance.note
                    }
                    events.push(eventData);
                }
            }
        });
        eventMap.set(gray, events);
        return events;
    } else {
        return eventMap.get(gray);
    }
}

// 获取配置表名
export const getCfgName = (sourceCfgName: string, gray: string): string => {
    return sourceCfgName + "/" + gray;
}
