/**
 * 数据库管理
 */

import { Env } from "pi_ucore/db/env";
import { Log } from "pi_pt/util/log";
import { Select, DbItem, DataR, Data } from "management_page/server/rpc/db_mgr.struct";
import { BonBuffer } from "pi_serialization/bon";
import { checkAuth } from "./user.event";
import { AUTH, FM, RESULT_NUM } from "../../../constant";
// import { AUTH, FM } from "management_page/server/db/user_db.struct";
import { DBType, Item } from "pi_ucore/db/mgr";
import { Tr } from "pi_pt/db/mgr";
import { HttpRequest, HttpResponse } from 'pi_pt/init/httprpc'
import { TabDecode, TabEncode } from "pi_serialization/tab_meta";
import { TabMeta } from "pi_serialization/sinfo";
import { checkHttpBasicAuth } from "./auth_lib";
import { checkHttpPortAuth, getEnv } from "./http_proxy.event";

declare var env: Env;

/**
 * 获取表数据 
 */
// #[httpRpc=/mgr/db/read]
export const readPort = async (req: HttpRequest, resp: HttpResponse) => {
    // 检测授权信息
    // checkHttpBasicAuth(req, resp);
    let userData = checkHttpPortAuth(req);
    let localEnv = getEnv(userData.user);

    let data = req.getData();
    let tab = data.tab;
    let key = data.key;
    console.log({ tab, key });
    let keyB = (await itemEncode(tab, key)).key;
    console.log("keyB:", keyB);
    let item = new DbItem();
    item.tab = tab;
    item.ware = "";
    item.key = keyB;
    return dataRDecode(tab, await read(item, localEnv));
}

// #[rpc=rpcServer, src=*]
export const read = async (table: DbItem, env: Env) => {
    if (!checkAuth(FM.DB_MGR, AUTH.R, env)) throw "权限错误";
    let result = new DataR();
    result.datas = [];
    let key = (await itemDecode(table.tab, table.key)).key;
    if(!Array.isArray(key)){
        key = new BonBuffer(table.key).read();
    }
    console.log('read', table.tab, key.constructor.name, key)
    await env.dbMgr.read(async (tr) => {
        let r = (await tr.query([{ ware: table.ware as DBType, tab: table.tab, key }], 1000, false))[0];
        let data = new Data();
        console.log('read vaule', r.key, r.value)
        let _d = await itemEncode(table.tab, r.key, r.value)
        data.key = _d.key;
        data.value = _d.value;
        result.datas.push(data);
    });
    Log.info("db_mgr read tab ", table);

    return result;
};

/**
 * 遍历表数据 
 */
// #[httpRpc=/mgr/db/select]
export const selectPort = async (req: HttpRequest, resp: HttpResponse) => {
    // 检测授权信息
    // checkHttpBasicAuth(req, resp);
    let userData = checkHttpPortAuth(req);
    let localEnv = getEnv(userData.user);

    let data = req.getData();
    let tab = data.tab;
    let key = data.key;
    let page_count = data.page_count ? data.page_count : 20;
    let descending = data.descending == "1" ? true : false;
    console.log({ tab, key });
    let item = new DbItem();
    item.ware = "";
    item.tab = tab;
    item.key = key ? (await itemEncode(tab, key)).key : key;
    let selectArg = new Select();
    selectArg.item = item;
    selectArg.page_count = page_count;
    selectArg.descending = descending as boolean;
    return dataRDecode(tab, await select(selectArg, localEnv));
}
// #[rpc=rpcServer, src=*]
export const select = async (range: Select, env: Env) => {
    if (!checkAuth(FM.DB_MGR, AUTH.R, env)) throw "权限错误";
    let result = new DataR();
    result.datas = [];
    let key;
    if (range.item.key) {
        key = (await itemDecode(range.item.tab, range.item.key)).key;
        if(!Array.isArray(key)){
            key = new BonBuffer(range.item.key).read();
        }
    }
    await env.dbMgr.read(async (tr) => {
        let iter = tr.kvList(range.item.ware as DBType, range.item.tab, key, range.descending, '');
        let count = range.page_count;
        for await (const item of iter) {
            if (key !== undefined && key !== null && item[0] === key) {
                // 跳过重复key
                continue;
            }
            let data = new Data();
            data.key = new BonBuffer().write(item[0]).getBuffer();
            data.value = new BonBuffer().write(item[1]).getBuffer();
            result.datas.push(data);
            count--;
            if (count < 1) break;
        }
    })
    Log.info("db_mgr select ", range);
    return result;
}

/**
 * 插入/修改/删除表数据 
 */
// #[httpRpc=/mgr/db/write]
export const writePort = async (req: HttpRequest, resp: HttpResponse) => {
    // 检测授权信息
    // checkHttpBasicAuth(req, resp);
    let userData = checkHttpPortAuth(req);
    let localEnv = getEnv(userData.user);

    let data = req.getData();
    let tab = data.tab;
    let key = data.key;
    let value = data.value ? JSON.parse(data.value) : undefined;
    console.log({ tab, key, value });
    let itemB = await itemEncode(tab, key, value);
    let item = new DbItem();
    item.tab = tab;
    item.ware = "";
    item.key = itemB.key;
    item.value = itemB.value;
    return await write(item, localEnv);
}
// #[rpc=rpcServer, src=*]
export const write = async (table: DbItem, env: Env) => {
    console.log("write table:", table);
    if (!checkAuth(FM.DB_MGR, AUTH.W, env)) throw "权限错误";
    let key = (await itemDecode(table.tab, table.key)).key;
    if(!Array.isArray(key)){
        key = new BonBuffer(table.key).read();
    }
    let value;
    if (!(table.value === undefined || table.value === null)) {
        value = new BonBuffer(table.value).read();
    };
    await env.dbMgr.write(async (tr) => {
        await tr.modify([{ ware: table.ware as DBType, tab: table.tab, key, value }], 1000, true);
    })
    Log.info("db_mgr write ", table);
    return true;
}

/**
 * 获取所有表名
 */
// #[httpRpc=/mgr/db/getAllTables]
export const getAllTablesPort = async (req: HttpRequest, resp: HttpResponse) => {
    // 检测授权信息
    // checkHttpBasicAuth(req, resp);
    let userData = checkHttpPortAuth(req);
    let localEnv = getEnv(userData.user);
    return await getAllTables("", localEnv);
}
// #[rpc=rpcServer, src=*]
export const getAllTables = async (ware: string, env: Env) => {
    if (!checkAuth(FM.DB_MGR, AUTH.R, env)) throw "权限错误";
    let tabs, r = [];
    tabs = await env.dbMgr.list_ware_tabs();
    for (const tab of tabs) {
        let item = new DbItem();
        item.ware = ware;
        item.tab = tab;
        r.push({ tab, info: await getTableInfo(item, env), size: await getTableSize(item, env) });
    }
    return r;
}

/**
 * 获取原信息
 */
// #[rpc=rpcServer, src=*]
export const getTableInfo = async (info: DbItem, env: Env) => {
    if (!checkAuth(FM.DB_MGR, AUTH.R, env)) throw "权限错误";
    try {
        return await env.dbMgr.read(async tr => {
            return JSON.stringify(await tr.tabInfo(info.tab));
        })
    } catch (error) {
        return error.message;
    }
}

/**
 * 获取表记录数
 */
// #[rpc=rpcServer, src=*]
export const getTableSize = async (tab: DbItem, env: Env) => {
    if (!checkAuth(FM.DB_MGR, AUTH.R, env)) throw "权限错误";
    return await env.dbMgr.tab_size(tab.tab);
}

type ItemObj = {
    ware: string,
    tab: string,
    key: string,
    value?: any
}
// 序列化item
const itemEncode = async (tab: string, key: any, value?: any) => {
    let meta: TabMeta, keyB: Uint8Array, valueB: Uint8Array;
    await env.dbMgr.read(async tr => {
        meta = await tr.tabInfo(tab);
    })
    let kBon = new BonBuffer();
    new TabEncode(meta.k, key).bonEncode(kBon);
    keyB = kBon.getBuffer();
    if (!(value === null || value === undefined)) {
        let vBon = new BonBuffer();
        new TabEncode(meta.v, value).bonEncode(vBon);
        valueB = vBon.getBuffer();
    }
    return { tab, key: keyB, value: valueB };
}

// 反序列化item
const itemDecode = async (tab: string, keyB: Uint8Array, valueB?: Uint8Array) => {
    let meta: TabMeta, key: any, value: any;
    await env.dbMgr.read(async tr => {
        meta = await tr.tabInfo(tab);
    })
    key = new TabDecode(meta.k).bonDecode(new BonBuffer(keyB));

    if (!(valueB === null || valueB === undefined)) {
        value = new TabDecode(meta.v).bonDecode(new BonBuffer(valueB));
    }
    return { tab, key, value }
}

// 处理dataR
const dataRDecode = async (tab: string, dataR: DataR) => {
    let meta: TabMeta, key: any, value: any, r = [];
    await env.dbMgr.read(async tr => {
        meta = await tr.tabInfo(tab);
    })
    for (const data of dataR.datas) {
        key = data.key ? new TabDecode(meta.k).bonDecode(new BonBuffer(data.key)) : data.key;
        value = data.value ? new TabDecode(meta.v).bonDecode(new BonBuffer(data.value)) : data.value;
        r.push({ key, value })
    }
    return r
}